"use strict";
var fs = require("fs");
var path = require("path");
var util_1 = require("./util");
var child_process_1 = require("child_process");
var codeSign_1 = require("./codeSign");
var merge = require("merge");
var packager = require("electron-packager");
var series = require("run-series");
var parallel = require("run-parallel");
function setDefaultOptionValues(options) {
    if (options.arch == null) {
        options.arch = "all";
    }
    if (options.platform == null) {
        options.platform = process.platform;
    }
}
exports.setDefaultOptionValues = setDefaultOptionValues;
var Packager = (function () {
    function Packager(options) {
        if (options === void 0) { options = {}; }
        this.options = options;
        this.isTwoPackageJsonProjectLayoutUsed = true;
        setDefaultOptionValues(options);
        this.appDir = this.computeAppDirectory();
        this.appPackageFile = path.join(this.appDir, "package.json");
        this.metadata = util_1.readPackageJson(this.appPackageFile);
        this.checkMetadata();
        this.distDir = path.join(process.cwd(), "dist");
        this.outDir = this.computeOutDirectory();
        console.log("Removing %s", this.outDir);
        require("rimraf").sync(this.outDir);
        var tasks = [];
        var cleanupTasks = [];
        var isMac = this.isMac;
        var keychainTaskAdded = false;
        var codeSigningInfo = null;
        var archs = isMac ? ["x64"] : (options.arch == null || options.arch === "all" ? ["ia32", "x64"] : [options.arch]);
        var _loop_1 = function(arch) {
            var distPath = path.join(this_1.outDir, this_1.metadata.name + (isMac ? ".app" : "-win32-" + arch));
            if (process.env.CSC_LINK != null && process.env.CSC_KEY_PASSWORD != null) {
                var packTask = this_1.pack.bind(this_1, arch, null);
                if (keychainTaskAdded) {
                    tasks.push(packTask);
                }
                else {
                    keychainTaskAdded = true;
                    tasks.push(util_1.parallelTask(packTask, function (callback) {
                        codeSign_1.createKeychain(util_1.handler(callback, function (result) { codeSigningInfo = result; }));
                    }));
                    cleanupTasks.push(codeSign_1.deleteKeychain);
                }
                util_1.addTasks(tasks, function (callback) {
                    console.log("Signing app");
                    child_process_1.execFile("codesign", ["--deep", "--force", "--sign", codeSigningInfo.cscName, distPath, "--keychain", codeSigningInfo.cscKeychainName], callback);
                });
            }
            else {
                tasks.push(this_1.pack.bind(this_1, arch, this_1.options.sign || process.env.CSC_NAME));
            }
            if (options.dist) {
                var distTask = this_1.packageInDistributableFormat.bind(this_1, arch, distPath);
                if (isMac) {
                    tasks.push(util_1.parallelTask(distTask, this_1.zipMacApp.bind(this_1)));
                }
                else {
                    tasks.push(distTask);
                }
                tasks.push(this_1.adjustDistLayout.bind(this_1, arch));
            }
        };
        var this_1 = this;
        for (var _i = 0, archs_1 = archs; _i < archs_1.length; _i++) {
            var arch = archs_1[_i];
            _loop_1(arch);
        }
        series(tasks, function (error) {
            parallel(cleanupTasks, function (cleanupError) {
                if (error == null) {
                    error = cleanupError;
                }
                if (error != null) {
                    if (typeof error === "string") {
                        console.error(error);
                    }
                    else if (error.message == null) {
                        console.error(error, error.stack);
                    }
                    else {
                        console.error(error.message);
                    }
                    process.exit(1);
                }
            });
        });
    }
    Object.defineProperty(Packager.prototype, "isMac", {
        get: function () {
            return this.options.platform === "darwin";
        },
        enumerable: true,
        configurable: true
    });
    Packager.prototype.computeAppDirectory = function () {
        var customAppPath = this.options.appDir;
        var required = true;
        if (customAppPath == null) {
            customAppPath = util_1.DEFAULT_APP_DIR_NAME;
            required = false;
        }
        var absoluteAppPath = path.join(process.cwd(), customAppPath);
        try {
            fs.accessSync(absoluteAppPath);
        }
        catch (e) {
            if (required) {
                throw new Error(customAppPath + " doesn't exists, " + e.message);
            }
            else {
                this.isTwoPackageJsonProjectLayoutUsed = false;
                return process.cwd();
            }
        }
        return absoluteAppPath;
    };
    Packager.prototype.computeOutDirectory = function () {
        var relativeDirectory;
        if (this.isMac) {
            relativeDirectory = this.metadata.name + "-darwin-x64";
        }
        else {
            relativeDirectory = "win";
        }
        return path.join(this.distDir, relativeDirectory);
    };
    Packager.prototype.checkMetadata = function () {
        var _this = this;
        var reportError = function (missedFieldName) {
            throw new Error("Please specify '" + missedFieldName + "' in the application package.json ('" + _this.appPackageFile + "')");
        };
        var metadata = this.metadata;
        if (metadata.name == null) {
            reportError("name");
        }
        else if (metadata.description == null) {
            reportError("description");
        }
        else if (metadata.version == null) {
            reportError("version");
        }
        else if (metadata.build == null) {
            throw new Error("Please specify 'build' configuration in the application package.json ('" + this.appPackageFile + "'), at least\n\n" +
                '\t"build": {\n' +
                '\t  "app-bundle-id": "your.id",\n' +
                '\t  "app-category-type": "your.app.category.type"\n'
                + '\t}' + "\n\n is required.\n");
        }
        else if (metadata.author == null) {
            reportError("author");
        }
    };
    Packager.prototype.pack = function (arch, macCscName, callback) {
        if (this.isTwoPackageJsonProjectLayoutUsed) {
            util_1.installDependencies(arch);
        }
        else {
            console.log("Skipping app dependencies installation because dev and app dependencies are not separated");
        }
        packager(merge({
            dir: this.appDir,
            out: this.options.platform === "win32" ? path.join(this.distDir, "win") : this.distDir,
            name: this.metadata.name,
            platform: this.options.platform,
            arch: arch,
            version: util_1.packageJson.devDependencies["electron-prebuilt"].substring(1),
            icon: path.join(process.cwd(), "build", "icon"),
            asar: true,
            "app-version": this.metadata.version,
            "build-version": this.metadata.version,
            sign: macCscName,
            "version-string": {
                CompanyName: this.metadata.author,
                FileDescription: this.metadata.description,
                FileVersion: this.metadata.version,
                ProductVersion: this.metadata.version,
                ProductName: this.metadata.name,
                InternalName: this.metadata.name,
            }
        }, this.metadata.build), callback);
    };
    Packager.prototype.zipMacApp = function (callback) {
        console.log("Zipping app");
        var appName = this.metadata.name;
        child_process_1.spawn("zip", ["-ryXq", (this.outDir + "/" + appName + "-" + this.metadata.version + "-mac.zip"), appName + ".app"], {
            cwd: this.outDir,
            stdio: "inherit",
        })
            .on("close", function (exitCode) {
            console.log("Finished zipping app");
            callback(exitCode === 0 ? null : "Failed, exit code " + exitCode);
        });
    };
    Packager.prototype.packageInDistributableFormat = function (arch, distPath, callback) {
        if (this.options.platform == "win32") {
            require('electron-installer-squirrel-windows')(merge({
                name: this.metadata.name,
                path: distPath,
                product_name: this.metadata.name,
                out: path.join(this.outDir, arch),
                version: this.metadata.version,
                description: this.metadata.description,
                authors: this.metadata.author,
                setup_icon: path.join(process.cwd(), "build", "icon.ico"),
            }, this.metadata.windowsPackager || {}), callback);
        }
        else {
            require("electron-builder").init().build(merge({
                "appPath": distPath,
                "platform": this.isMac ? "osx" : this.options.platform,
                "out": this.outDir,
                "config": path.join(process.cwd(), "build", "packager.json"),
            }, this.metadata.darwinPackager || {}), callback);
        }
    };
    Packager.prototype.adjustDistLayout = function (arch, callback) {
        var appName = this.metadata.name;
        if (this.options.platform === "darwin") {
            fs.rename(path.join(this.outDir, appName + ".dmg"), path.join(this.outDir, appName + "-" + this.metadata.version + ".dmg"), callback);
        }
        else {
            fs.rename(path.join(this.outDir, arch, appName + "Setup.exe"), path.join(this.outDir, appName + "Setup-" + this.metadata.version + ((arch === "x64") ? "-x64" : "") + ".exe"), callback);
        }
    };
    return Packager;
}());
exports.Packager = Packager;
//# sourceMappingURL=build.js.map