"use strict";
var fs = require("fs");
var path = require("path");
var series = require("run-series");
var parallel = require("run-parallel");
var merge = require("merge");
exports.packageJson = readPackageJson(path.join(process.cwd(), "package.json"));
exports.DEFAULT_APP_DIR_NAME = "app";
exports.commonArgs = [
    {
        name: "appDir",
        type: String,
        description: "Relative (to the working directory) path to the folder containing the application package.json. Working directory or app/ by default."
    }
];
function reportResult(result) {
    if (result.status != 0) {
        if (result.error != null) {
            console.error(result.error);
        }
        process.exit(result.status);
    }
}
exports.reportResult = reportResult;
function readPackageJson(path) {
    try {
        return JSON.parse(fs.readFileSync(path));
    }
    catch (e) {
        if (e instanceof SyntaxError) {
            console.error(path + " is not a valid JSON file");
        }
        throw e;
    }
}
exports.readPackageJson = readPackageJson;
function installDependencies(arch, appDir) {
    if (appDir == null) {
        appDir = exports.DEFAULT_APP_DIR_NAME;
    }
    var processWorkingDirectory = path.join(process.cwd(), appDir);
    console.log("Installing production dependencies for arch " + (arch || process.arch) + " to " + processWorkingDirectory);
    var electronPrebuiltDep = exports.packageJson.devDependencies["electron-prebuilt"];
    if (electronPrebuiltDep == null) {
        throw new Error("Cannot find electron-prebuilt dependency to get electron version");
    }
    var env = merge(true, process.env, {
        npm_config_disturl: "https://atom.io/download/atom-shell",
        npm_config_target: electronPrebuiltDep.substring(1),
        npm_config_runtime: "electron",
        HOME: require("os").homedir() + "/.electron-gyp",
    });
    if (arch != null) {
        env.npm_config_arch = arch;
    }
    var npmExecPath = process.env.npm_execpath || process.env.NPM_CLI_JS;
    var npmExecArgs = ["install"];
    if (npmExecPath == null) {
        npmExecPath = "npm";
    }
    else {
        npmExecArgs.unshift(npmExecPath);
        npmExecPath = (process.env.npm_node_execpath || process.env.NODE_EXE || process.env.NODE_EXE || "node");
    }
    reportResult(require("child_process").spawnSync(npmExecPath, npmExecArgs, {
        cwd: processWorkingDirectory,
        stdio: "inherit",
        env: env
    }));
}
exports.installDependencies = installDependencies;
function handler(callback, handler) {
    return function (error, result) {
        if (error == null) {
            try {
                handler(result);
            }
            catch (e) {
                callback(e);
                return;
            }
            callback(null);
        }
        else {
            callback(error);
        }
    };
}
exports.handler = handler;
function parallelTask() {
    var tasks = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        tasks[_i - 0] = arguments[_i];
    }
    return parallel.bind(null, tasks);
}
exports.parallelTask = parallelTask;
function seriesTask() {
    var tasks = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        tasks[_i - 0] = arguments[_i];
    }
    return series.bind(null, tasks);
}
exports.seriesTask = seriesTask;
function createTasks() {
    var callbacks = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        callbacks[_i - 0] = arguments[_i];
    }
    return callbacks;
}
function addTasks(target) {
    var tasks = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        tasks[_i - 1] = arguments[_i];
    }
    target.push.apply(target, tasks);
}
exports.addTasks = addTasks;
function task(dependencies, task) {
    if (dependencies == null) {
        return [task];
    }
    var result = Array(dependencies.length + 1);
    result.push.apply(result, dependencies);
    result.push(task);
    return result;
}
//# sourceMappingURL=util.js.map